﻿/********************************************************
*********************************************************
**                                                                                                          **
** 	PIXIE ARMY v 0.9                                                                        **
** 	Written by Massimiliano Pareschi for Bonsaininja Studio                  **
**  	Free to redistibute                                                                        **
** 	Use of code is free upon request to the author and proper cretiting  **
** 	If you need anithing write to thejaist@gmail.com                               **
**                                                                                                          **
*********************************************************
*********************************************************/

{
	/*-------------------------------------------------------------------------------------*/
	function initPixieDialog(UI)
	/*-------------------------------------------------------------------------------------*/	
	{

		UI.dlg = new Window("palette", "::Pixie Army::", [0, 0, 200, 410]);
		
		UI.dlg.add("image",[0,0,200,45], "pixie_army.png");
		
		UI.dlg.dimensionsPanel = UI.dlg.add("panel", [10,50,190,172], "Grid Dimensions");
		
		UI.dlg.dimensionsPanel.add("statictext",[10,20,70,49],"Rows:");
		UI.dlg.dimensionsPanel.add("statictext",[10,50,70,79],"Columns:");
		UI.dlg.dimensionsPanel.add("statictext",[10,80,70,109],"Spacing:");
		
		UI.dlg.plate_rows = UI.dlg.dimensionsPanel.add("edittext", [70,17,160,37],"16");
		UI.dlg.plate_columns = UI.dlg.dimensionsPanel.add("edittext", [70,47,160,67],"9");
		UI.dlg.plate_spacing = UI.dlg.dimensionsPanel.add("edittext", [70,77,160,97],"10");
		
		UI.dlg.optionsPanel = UI.dlg.add("panel", [10,180,190,340], "Options");
		
		UI.dlg.optionsPanel.add("statictext",[10,20,70,49],"Accuracy:");
		UI.dlg.optionsPanel.add("statictext",[10,50,70,79],"Displace:");
		
		UI.dlg.sampling_precision = UI.dlg.optionsPanel.add("dropdownlist", [70,17,160,37],["Full","Half","Third","Quarter"]);
		UI.dlg.sampling_precision.selection = 0;
		
		UI.dlg.maxDisplace = UI.dlg.optionsPanel.add("edittext", [70,47,160,67],"300");
		UI.dlg.HDRClamp = UI.dlg.optionsPanel.add("checkbox",[8,77,160,96], "Clip HDR values");
		UI.dlg.zDirection = UI.dlg.optionsPanel.add("checkbox",[8,97,160,116], "Negative Z Displace");
		UI.dlg.fastMode = UI.dlg.optionsPanel.add("checkbox",[8,117,160,136],"Safe | Fast Mode");
		
		UI.dlg.pgrBar = UI.dlg.add("progressbar",[10,345,190,355]);
		
		UI.dlg.startButton = UI.dlg.add("button",[10,365,190,400],"Go Pixie Go!");
		
		UI.dlg.startButton.onClick = goPixieGo_onClick;

	}
	/*-------------------------------------------------------------------------------------*/
	function goPixieGo_onClick()
	/*-------------------------------------------------------------------------------------*/
	{
		var rows = parseInt(UI.dlg.plate_rows.text);
		var columns = parseInt(UI.dlg.plate_columns.text);
		var spacing = parseInt(UI.dlg.plate_spacing.text);
		var sampling = 1;
		switch (UI.dlg.sampling_precision.selection.index)
		{
			case 0:
				sampling = 1;
				break;
			case 1:
				sampling = 0.75;
				break;
			case 2:
				sampling = 0.50;
				break;
			case 3:
				sampling = 0.25;
				break;
		}
		var fmode = UI.dlg.fastMode.value;
		var displace = parseInt(UI.dlg.maxDisplace.text);
		var clamp = UI.dlg.HDRClamp.value;
		var direction = UI.dlg.zDirection.value;
		var pgrbar = UI.dlg.pgrBar;
		doPixieArmy(rows,columns,spacing,sampling,displace,clamp,direction,fmode,pgrbar);
	}
	/*-------------------------------------------------------------------------------------*/
	function buildLayerList(myComp)
	/*-------------------------------------------------------------------------------------*/
	{
		// Returns a collelction with all the layers in the comp
		// Designed to solve the ambiguity in collections enumerating, the returned list has a starting index of 0 instead of 1
		
		var activeItem= myComp;
		var allLayers = new Array();
		for (var i = 1; i <= activeItem.layers.length;i++)
		{
			allLayers.push(activeItem.layers[i]);
		}
		return allLayers;
	}

	/*-------------------------------------------------------------------------------------*/
	function buildAVLayerList(myComp, AUTOSELECT_FLAG)
	/*-------------------------------------------------------------------------------------*/
	{  // when flag is   ALL  autoselects all layers in a comp
		// Finds the selected layers and populates a collection
		// If nothing is selected AND the flag value is ALL then autoselects everything
		// then strips from the selection unwanted types: Cameras, Lights and Adjustment Layers
		// Returns a collection with all the AV and Text Layers in the comp

		var activeItem= myComp;
		var allLayers = activeItem.selectedLayers;
		if (allLayers.length == 0 && AUTOSELECT_FLAG == "ALL")
		{
			allLayers = buildLayerList(activeItem);
		}
		if (allLayers.length>0)
		{
			var AVLayers = new Array();
			for (var i = 0; i < allLayers.length; i++)
			{
				if ((allLayers[i].matchName == "ADBE AV Layer" || allLayers[i].matchName == "ADBE Text Layer") && allLayers[i].adjustmentLayer == false)
				{
					AVLayers.push(allLayers[i]);
				}
			}
		}
		else
		{
			var AVLayers = false;
		}
		return AVLayers;
	}

	/*-------------------------------------------------------------------------------------*/
	function buildCameraLayers(myComp)
	/*-------------------------------------------------------------------------------------*/
	{
		// Finds all the cameras in the scene and returns a list containing all of them
		// If the list is empty then returns false

		var activeItem= myComp;
		var compCams = new Array();
			for (var i = 1; i <= activeItem.layers.length;i++)
			{
				if (activeItem.layers[i].matchName == "ADBE Camera Layer")
				{
					compCams.push(activeItem.layers[i]);
				}
			}
		if (compCams.length == 0)
		{
			return false;
		}
		else
		{
			return compCams;
		}
	}

	/*-------------------------------------------------------------------------------------*/
	function cameraConform(myComp,cameraList)
	/*-------------------------------------------------------------------------------------*/
	{
		var activeItem= myComp;
		if (cameraList != false)
		{
			for (var i = 0; i < cameraList.length;i++)
			{
				cameraList[i].name = "old_camera_0"+i;
				cameraList[i].enabled = false;
			}
		}
		var newCamera = activeItem.layers.addCamera("PIX_camera", [myComp.width/2,myComp.height/2]);
		newCamera.selected = false;
		return newCamera;
	}
	
	/*-------------------------------------------------------------------------------------*/
	function doPixieArmy(rows,columns,spacing,sampling,displace,clamp,direction,FAST_MODE_FLAG,pgrbar)
	/*-------------------------------------------------------------------------------------*/
	{
		
		activeItem = app.project.activeItem;
		undoString = "Apply Pixie Army";
		
		app.beginUndoGroup(undoString);
		
		if (activeItem == null || !(activeItem instanceof CompItem))
		{
			alert("Please establish a comp as the active item and run the script again");	
		}
		else
		{
			var AVLayers = buildAVLayerList(activeItem);
			var cameraLayers = buildCameraLayers(activeItem);
			
			if (AVLayers.length > 1 || AVLayers == false)
			{
				alert("Please select ONLY ONE layer.");			
			}
			else
			{
					
				//______________________________________________________________________________________________________________
				//______________________________________________________________________________________________________________
				//
				//                                                                           				LOCAL VARIABLES DEFINITION
				//______________________________________________________________________________________________________________
				//______________________________________________________________________________________________________________
				

				var plate = AVLayers[0];
				var cameraFocal = 35;
				var cameraZoom = cameraFocal*7.528*2.834;   // Camera FOCAL to ZOOM multiplier = 7.528
				var cameraAperture = cameraFocal*0.178*2.834;   // Camera MILLIMETERS to PIXELS multiplier = 2.834
				var cameraOffset = 100;
				
				plate.threeDLayer = true;
				plate.enabled = false;
				
				var plate_rows = rows; 
				var plate_columns = columns;
				var plate_spacing = spacing;    //  PIXELS OF SPACING BETWEEN THE SOLIDS
				var sampling_precision = sampling;    // AREA OF THE SAMPLEINFO SEARCH, 0 TO 1
				
				var solidSourceColor = [1.0,0.3,0.0];
				
				var myComponent = 0.0;  // BASE COLOR -- BLACK
				var HDRClamp = clamp; // CLIPS VALUES AT THIS CEILING
				var maxDisplace = displace;  // AMOUNT TO Z-SHIFT
				if (!FAST_MODE_FLAG)
				{
					var zDirection = direction;
				}
				else
				{
					var zDirection = 1;
					direction == true ? zDirection = 1 : zDirection = -1;
				}

				//______________________________________________________________________________________________________________
				//______________________________________________________________________________________________________________
				
				
				//  CREATE A NEW CAMERA WITH DEFAULT SETTINGS
				
				var myNewPixieCamera = cameraConform(activeItem, cameraLayers);
				myNewPixieCamera.zoom.setValue(cameraZoom);
				myNewPixieCamera.focusDistance.setValue(cameraZoom);
				myNewPixieCamera.position.setValue([activeItem.width/2,activeItem.height/2,-cameraZoom]);
				myNewPixieCamera.aperture.setValue(cameraAperture);			
				
				//  CREATE A NEW CONTROLLER NULL
				
				var myNewPixieController = activeItem.layers.addNull();
				myNewPixieController.source.name = "Pixie Controller";
				myNewPixieController.position.setValue([activeItem.width/2,activeItem.height/2,0]);
				plate.parent = myNewPixieController;
				myNewPixieController.threeDLayer = true;
				myNewPixieController.name = "Pixie_Controller";
				if (!FAST_MODE_FLAG)
				{
					myNewPixieController.Effects.addProperty("ADBE Slider Control");
					myNewPixieController.Effects.addProperty("ADBE Checkbox Control");
					myNewPixieController.Effects.addProperty("ADBE Checkbox Control");
					myNewPixieController.effect(1).name = "Max Displacement";
					myNewPixieController.effect(2).name = "HDR Clamp Values";
					myNewPixieController.effect(3).name = "Negative Z Displacement";
					myNewPixieController.effect(1).property(1).setValue(maxDisplace);
					myNewPixieController.effect(2).property(1).setValue(HDRClamp);
					myNewPixieController.effect(3).property(1).setValue(zDirection);	
				}
			
				// CREATES THE MASTER SOLIDSOURCE 
				
				var myNewTempLayer = activeItem.layers.addSolid(solidSourceColor,"Pixie Solid",Math.floor(plate.width/plate_rows)-plate_spacing,Math.floor(plate.height/plate_columns)-plate_spacing,1);
				var myNewPixieSolidSource = myNewTempLayer.source;
				myNewTempLayer.remove();
				
				//  SETS EXPRESSIONS AS STRING VARIABLES
				
				if (!FAST_MODE_FLAG)
				{
					var colorExpression = "colormap = thisComp.layer(\""+plate.name+"\");\r"+
					"parentPosition = thisComp.layer(\""+myNewPixieController.name+"\").transform.position;\r"+
					"thisPosition = thisLayer.transform.position;\r"+
					"sampleLocation = [parentPosition[0]+thisPosition[0],parentPosition[1]+thisPosition[1]];\r"+
					"myColor = colormap.sampleImage(sampleLocation,[(thisLayer.width/"+sampling_precision+"),(thisLayer.height/"+sampling_precision+")], true, time);\r"+
					"myColor[3] = 1.0;\r"+
					"myColor;"					
				}
				else
				{
					var colorExpression = "colormap = thisComp.layer(\""+plate.name+"\");\r"+
					"sampleLocation = ["+(activeItem.width/2)+"+transform.position[0],"+(activeItem.height/2)+"+transform.position[1]];\r"+
					"myColor = colormap.sampleImage(sampleLocation,[(thisLayer.width/"+sampling_precision+"),(thisLayer.height/"+sampling_precision+")], true, time);\r"+
					"myColor[3] = 1.0;\r"+
					"myColor;"	
				}

				var colorDerivedExp = "effect(\"colorOffset\")(\"Color\")";

				if (!FAST_MODE_FLAG)
				{
					var positionExpression = "maxDisplace = thisComp.layer(\"Pixie_Controller\").effect(\"Max Displacement\")(\"Slider\");\r"+
					"zDirection =  thisComp.layer(\"Pixie_Controller\").effect(\"Negative Z Displacement\")(\"Checkbox\");\r"+
					"zDirection == true ? zDirection = 1.0 : zDirection = -1.0;\r"+
					"baseColor = effect(\"colorOffset\")(\"Color\");\r"+
					"lumaValue = (baseColor[0]+baseColor[1]+baseColor[2])/3;\r"+
					"basePosition = thisLayer.transform.position;\r"+
					"[basePosition[0], basePosition[1], basePosition[2]+(lumaValue*(maxDisplace*zDirection))]\r";
				}
				else
				{					
					var positionExpression = "baseColor = effect(\"colorOffset\")(\"Color\");\r"+
					"[transform.position[0], transform.position[1], transform.position[2]+(((baseColor[0]+baseColor[1]+baseColor[2])/3)*("+maxDisplace+"*("+zDirection+")))]\r";
				}
				
				// CREATE PIXIE LAYERS AND CONTROLS
				
				for (var i = 0; i < plate_columns; i++)
				{
					for (var j = 0; j< plate_rows; j++)
					{
						
						// SOLID NAME DEFINITION

						var myCounter = i*plate_rows+j;
						var myName = ""+(myCounter+1)+"";

						switch (myName.length)
						{
							case 1 :
								myName = "000"+myName;
								break;
							case 2 :
								myName = "00"+myName;
								break;
							case 3 :
								myName = "0"+myName;
						}

						// CREATE LAYER AND REPOSITION
						
						var myNewPixieLayer = activeItem.layers.add(myNewPixieSolidSource);
						myNewPixieLayer.name = "Pixie Layer "+ myName;
						myNewPixieLayer.threeDLayer = true;
						myNewPixieLayer.motionBlur = true;
						myNewPixieLayer.shy = true;
						myNewPixieLayer.position.setValue([(plate.width/plate_rows)*j+(myNewPixieLayer.width/2)+(plate_spacing/2),(plate.height/plate_columns)*i+(myNewPixieLayer.height/2)+(plate_spacing/2),0]);
						myNewPixieLayer.parent = myNewPixieController;
						
						//CREATE EXPRESSIONS FOR COLOR SHIFTING AND POSITION DISPLACE
						
						myNewPixieLayer.Effects.addProperty("ADBE Color Control");
						myNewPixieLayer.Effects.property("ADBE Color Control").name = "colorOffset";
						myNewPixieLayer.Effects.property("colorOffset").property(1).expression = colorExpression;
						
						myNewPixieLayer.Effects.addProperty("ADBE Solid Composite");
						myNewPixieLayer.Effects.property("ADBE Solid Composite").name = "colorFill";
						myNewPixieLayer.Effects.property("colorFill").property(1).setValue(0.0);
						myNewPixieLayer.Effects.property("colorFill").property(2).expression = colorDerivedExp;
						
						myNewPixieLayer.position.expression = positionExpression;
						
						pgrbar.value = ((myCounter+1)*(plate_rows*plate_columns))/pgrbar.maxvalue;
						
					}
				}
	
				/// ADD CLAMPING LEVELS TO THE FUNK
				
				plate.Effects.addProperty("ADBE Easy Levels2");
				
				if (!FAST_MODE_FLAG)
				{
					plate.effect(1).property(8).expression = "thisComp.layer(\"Pixie_Controller\").effect(\"HDR Clamp Values\")(\"Checkbox\") == true ? 1 : 3;\r";
					plate.effect(1).property(9).expression = "thisComp.layer(\"Pixie_Controller\").effect(\"HDR Clamp Values\")(\"Checkbox\") == true ? 1 : 3;\r";
				}
				else
				{
					if (HDRClamp)
					{
						plate.effect(1).property(8).setValue(1);
						plate.effect(1).property(9).setValue(1);
					}
					else
					{
						plate.effect(1).property(8).setValue(3);
						plate.effect(1).property(9).setValue(3);
					}
				}
			
				myNewPixieController.moveToBeginning();
				myNewPixieCamera.moveToBeginning();
				
				activeItem.hideShyLayers = true;
				activeItem.motionBlur = false;

				plate.locked = true;
				
				app.endUndoGroup();
				
				pgrbar.value = 0;
			}
			
		}
	}

	var UI = new Object();
	initPixieDialog(UI);

	UI.dlg.show();
	UI.dlg.frameLocation = [100,100];

}